/* UtilLib_Html - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html.jsoup;

import net.mapeadores.util.html.TrustedHtmlFactory;
import org.jsoup.Jsoup;
import org.jsoup.safety.Whitelist;


/**
 *
 * @author Vincent Calame
 */
public class CleanTrustedHtmlFactory extends TrustedHtmlFactory {

    public final static CleanTrustedHtmlFactory NONE = new CleanTrustedHtmlFactory(Whitelist.none());
    public final static CleanTrustedHtmlFactory SIMPLE_TEXT = new CleanTrustedHtmlFactory(Whitelist.simpleText());
    public final static CleanTrustedHtmlFactory BASIC = new CleanTrustedHtmlFactory(Whitelist.basic());
    public final static CleanTrustedHtmlFactory BASIC_WITH_IMAGES = new CleanTrustedHtmlFactory(Whitelist.basicWithImages());
    public final static CleanTrustedHtmlFactory RELAXED = new CleanTrustedHtmlFactory(Whitelist.relaxed());
    public final static CleanTrustedHtmlFactory EXTENDED = new CleanTrustedHtmlFactory(extended());
    public final static CleanTrustedHtmlFactory EXTENDED_WITH_STYLE = new CleanTrustedHtmlFactory(extendedWithStyle());
    private final Whitelist whitelist;

    public CleanTrustedHtmlFactory(Whitelist whitelist) {
        this.whitelist = whitelist;
    }

    @Override
    public String check(String html) {
        return Jsoup.clean(html, whitelist);
    }

    public static Whitelist extended() {
        return Whitelist.relaxed()
                .addTags("hr", "iframe", "video", "audio", "source")
                .addAttributes("iframe", "src", "name", "allowfullscreen", "height", "width", "frameborder", "scrolling", "marginwidth", "marginheight")
                .addAttributes("video", "src", "crossorigin", "poster", "preload", "autoplay", "mediagroup", "loop", "muted", "controls", "height", "width")
                .addAttributes("audio", "src", "crossorigin", "preload", "autoplay", "mediagroup", "loop", "controls")
                .addAttributes("source", "src", "type")
                .addProtocols("iframe", "src", "http", "https")
                .addProtocols("video", "src", "http", "https")
                .addProtocols("audio", "src", "http", "https")
                .addProtocols("source", "src", "http", "https")
                .addAttributes(":all", "id", "class", "lang");
    }

    public static Whitelist extendedWithStyle() {
        return extended()
                .addAttributes(":all", "style");
    }


}
